@@ -1,8 +1,46 @@
 use crate::*;
-use anchor_lang::solana_program::sysvar::instructions::get_instruction_relative;
+use anchor_lang::solana_program::{system_program, sysvar::instructions::get_instruction_relative};
 use anchor_spl::token;
 use num_traits::ToPrimitive;
 
+/// Accounts for [locked_voter::lock].
+#[derive(Accounts)]
+pub struct Lock<'info> {
+    /// [Locker].
+    #[account(mut)]
+    pub locker: Account<'info, Locker>,
+
+    /// [Escrow].
+    #[account(mut)]
+    pub escrow: Account<'info, Escrow>,
+
+    /// Token account held by the [Escrow].
+    #[account(mut)]
+    pub escrow_tokens: Account<'info, TokenAccount>,
+
+    /// Authority of the [Escrow] and [Self::source_tokens].
+    pub escrow_owner: Signer<'info>,
+
+    /// The source of deposited tokens.
+    #[account(mut)]
+    pub source_tokens: Account<'info, TokenAccount>,
+
+    /// Token program.
+    pub token_program: Program<'info, Token>,
+}
+
+/// Accounts for [locked_voter::set_locker_params].
+#[derive(Accounts)]
+pub struct SetLockerParams<'info> {
+    /// The [Locker].
+    #[account(mut)]
+    pub locker: Account<'info, Locker>,
+    /// The [Governor].
+    pub governor: Account<'info, Governor>,
+    /// The smart wallet on the [Governor].
+    pub smart_wallet: Signer<'info>,
+}
+
 impl<'info> SetLockerParams<'info> {
     pub fn set_locker_params(&mut self, params: LockerParams) -> ProgramResult {
         let prev_params = self.locker.params;
@@ -32,9 +70,13 @@ impl<'info> Validate<'info> for SetLockerParams<'info> {
 
 impl<'info> Lock<'info> {
     pub fn lock(&mut self, amount: u64, duration: i64) -> ProgramResult {
+        invariant!(
+            unwrap_int!(duration.to_u64()) >= self.locker.params.min_stake_duration,
+            LockupDurationTooShort
+        );
         invariant!(
             unwrap_int!(duration.to_u64()) <= self.locker.params.max_stake_duration,
-            "max stake duration exceeded"
+            LockupDurationTooLong
         );
 
         // check that the escrow refresh is valid
@@ -50,7 +92,7 @@ impl<'info> Lock<'info> {
             );
             invariant!(
                 next_escrow_ends_at >= prev_escrow_ends_at,
-                "escrow refresh cannot shorten the escrow time remaining"
+                RefreshCannotShorten
             );
         }
 
@@ -93,10 +135,7 @@ impl<'info> Lock<'info> {
     }
 
     pub fn check_whitelisted(&self, ra: &[AccountInfo]) -> ProgramResult {
-        invariant!(
-            ra.len() == 2,
-            "program whitelist enabled; please provide whitelist entry and instructions sysvar"
-        );
+        invariant!(ra.len() == 2, MustProvideWhitelist);
         let accounts_iter = &mut ra.iter();
         let ix_sysvar_account_info = next_account_info(accounts_iter)?;
         let program_id = get_instruction_relative(0, ix_sysvar_account_info)?.program_id;
@@ -111,8 +150,15 @@ impl<'info> Lock<'info> {
         );
         let whitelist_entry =
             Account::<LockerWhitelistEntry>::try_from(whitelist_entry_account_info)?;
-        assert_keys_eq!(whitelist_entry.locker, self.locker.key());
+        assert_keys_eq!(whitelist_entry.locker, self.locker);
         assert_keys_eq!(whitelist_entry.program_id, program_id);
+        if whitelist_entry.owner != system_program::ID {
+            assert_keys_eq!(
+                whitelist_entry.owner,
+                self.escrow_owner,
+                EscrowOwnerNotWhitelisted
+            );
+        }
 
         Ok(())
     }
@@ -128,3 +174,40 @@ impl<'info> Validate<'info> for Lock<'info> {
         Ok(())
     }
 }
+
+#[event]
+/// Event called in [locked_voter::lock].
+pub struct LockEvent {
+    /// The locker of the [Escrow]
+    #[index]
+    pub locker: Pubkey,
+    /// The owner of the [Escrow].
+    #[index]
+    pub escrow_owner: Pubkey,
+    /// Mint of the token that for the [Locker].
+    pub token_mint: Pubkey,
+    /// Amount of tokens locked.
+    pub amount: u64,
+    /// Amount of tokens locked inside the [Locker].
+    pub locker_supply: u64,
+    /// Duration of lock time.
+    pub duration: i64,
+    /// The previous timestamp that the [Escrow] ended at.
+    pub prev_escrow_ends_at: i64,
+    /// The new [Escrow] end time.
+    pub next_escrow_ends_at: i64,
+    /// The new [Escrow] start time.
+    pub next_escrow_started_at: i64,
+}
+
+/// Event called in [locked_voter::set_locker_params].
+#[event]
+pub struct LockerSetParamsEvent {
+    /// The [Locker].
+    #[index]
+    pub locker: Pubkey,
+    /// Previous [LockerParams].
+    pub prev_params: LockerParams,
+    /// New [LockerParams].
+    pub params: LockerParams,
+}
